#!/usr/bin/ruby
#
# Copyright (c) 2001 Yasuhiro ABE <yasu@dengaku.org>
#
# Usage: make.rb [-t|-n] [-d directory]
#

require "kconv"

class OUTPUT
  def initialize
    @dev = $stdout
  end

  def set_dev(dev)
    @dev = dev
  end

  def print(string)
    return if string == nil
    @dev.print Kconv::tojis(string) + "\n"
  end
end


class INDEX
  attr_accessor :use_table
  attr_accessor :use_normal

  def initialize
    @out = OUTPUT.new
    @path_level = 0
    @path_list = Array.new
  end

  def check_essential_files(dir)
    result = Array.new(0)
    e_files = [ "head.inc", "index.inc", "tail.inc", "title.inc" ]
    e_files.each{|file|
      result << file if not FileTest.file?(dir + File::Separator + file)
    }
    return result
  end

  def make_fullpath(file)
    abs_path = @path_list[@path_list.size-1] + File::Separator + file
    return abs_path
  end

  def make_index_table(fullpath)
    result = ""
    title = nil
    prev_indent = 0
    open(fullpath).each{ |line|
      next if line =~ "^\n"
      next if line !~ /\S/
      if not title 
	title = line.chop 
	result = format "<H2>%s</H2>\n\n", title
	next
      end
      indent = line =~ /\S/
      
      if prev_indent < indent
	result += format("<UL>\n  <LI>%s", line)
      elsif prev_indent > indent
	result += format("</UL>\n")
	result += format("<BR>%s", line)
      elsif indent == 1
	result += format("  <LI>%s", line)
      else
	result += format("<BR>%s", line)
      end
      prev_indent = indent
    }

    return result
  end

  def make_index_normal(fullpath)
    result = ""
    title = nil
    tmp_list = Array.new
    is_first = false
    open(fullpath).each{ |line|
      next if line =~ "^\n"
      if not title 
	title = line.chop 
	result = format("<BR>%s: [ ", title)
	next
      end
      indent = line =~ /\S/

      if indent > 0
	tmp_list << line.chop
      else
	if not is_first
	  result += format(" %s ", line.chop)
	  is_first = true
	else
	  result += format(" | %s ", line.chop)
	end
      end
    }
    result += format("]\n")

    if tmp_list.size > 0
      is_first = nil
      result += format("<BR>[ ")
      tmp_list.each{ |item|
	if not is_first
	  result += format(" %s ", item)
	  is_first = true
	else
	  result += format(" | %s ", item)
	end
      }
      result += format(" ]\n")
    end

    return result
  end

  # run() method has the responsibility to manage the @path_level variable
  def run(dir)
    @path_level += 1
    @path_list << dir
    Dir.foreach(dir) { |file|
      next if file == "."
      next if file == ".."
      if FileTest.file?(make_fullpath(file)) and
	  make_fullpath(file) =~ /rhtml$/
	if @use_table
	  produce_table(file)
	elsif @use_normal
	  produce_normal(file)
	else
	  produce_normal(file)
	end
      end
      run(file) if FileTest.directory?(file)
    }
    @path_level -= 1
    @path_list.delete_at(@path_list.size-1)
  end

  # 1. the produce method should not change the @path_level variable
  def produce_table(file)
    fullpath = make_fullpath(file)
    # error check
    return if FileTest.file?(fullpath) == false
    
    # debug message
    printf "processing %s\n", fullpath

    # prepare output file
    out = OUTPUT.new
    out.set_dev(open(fullpath.sub(/rhtml$/,"html"),"w"))

    # begin html document
    out.print open(@path_list[0] + File::Separator + "head.inc").read.gsub("__YASU_PATH_","../" * (@path_level-1))
    out.print "<TABLE width=100% height=100% border=\"0\" cellspacing=\"0\" cellpadding=\"4\">"
    out.print " <TR>"
    out.print "   <TD class=index valign=top nowrap=true>"

    if FileTest.file?(make_fullpath("index.inc"))
      out.print make_index_table(make_fullpath("index.inc")).gsub("__YASU_PATH_","../" * (@path_level-1))
    else
      out.print make_index_table(@path_list[0] + File::Separator + "index.inc").gsub("__YASU_PATH_","../" * (@path_level-1))
    end

    out.print "   <TD class=index2>&nbsp;</TD>"
    out.print "   <TD class=main valign=top>"
    out.print "   <TD class=title colspan=3 valign=top>"
    if FileTest.file?(make_fullpath("title.inc"))
      out.print open(make_fullpath("title.inc")).read
    else
      out.print open(@path_list[0] + File::Separator + "title.inc").read
    end

    out.print open(fullpath).read
    out.print "   </TD>"
    out.print " </TR>"
    out.print "</TABLE>"

    out.print out.print open(@path_list[0] + File::Separator + "tail.inc").read
    # end of html
  end

  def produce_normal(file)
    dir = @path_list[@path_list.size-1]
    fullpath = dir + File::Separator + file

    # error check
    return if FileTest.file?(fullpath) == false
    
    # debug message
    printf "processing %s\n", fullpath

    # prepare output file
    out = OUTPUT.new
    out.set_dev(open(fullpath.sub(/rhtml$/,"html"),"w"))

    # begin html document
    out.print open(@path_list[0] + File::Separator + "head.inc").read.gsub("__YASU_PATH_","../" * (@path_level-1))

    # make index
    out.print "<DIV class=\"index\">"
    if FileTest.file?(dir + File::Separator + "index.inc")
      out.print make_index_normal(make_fullpath("index.inc")).gsub("__YASU_PATH_","../" * (@path_level-1))
    else
      out.print make_index_normal(@path_list[0] + File::Separator + "index.inc").gsub("__YASU_PATH_","../" * (@path_level-1))
    end
    out.print "</DIV><HR>"

    # title and main text
    if FileTest.file?(dir + File::Separator + "title.inc")
      out.print open(dir + File::Separator + "title.inc").read
    else
      out.print open(@path_list[0] + File::Separator + "title.inc").read
    end
    out.print open(fullpath).read

    # make index
    out.print "<HR><DIV class=\"index\">"
    if FileTest.file?(dir + File::Separator + "index.inc")
      out.print make_index_normal(make_fullpath("index.inc")).gsub("__YASU_PATH_","../" * (@path_level-1))
    else
      out.print make_index_normal(@path_list[0] + File::Separator + "index.inc").gsub("__YASU_PATH_","../" * (@path_level-1))
    end
    out.print "</DIV>"

    out.print out.print open(@path_list[0] + File::Separator + "tail.inc").read
    # end of html
  end
end

def usage
  printf "\nusage: make.rb [-t|-n] [-d directory]\n"
  printf "  -t: use table format\n"
  printf "  -n: use text (normal) format\n"
  printf "  -d: specify top directory\n"

  printf "\n"
end
$USAGE = "usage"

#
# MAIN
#
index = INDEX.new

require "parsearg"
parseArgs(0, nil, "tn", "d:.")

if $OPT_t
  index.use_table = true
elsif $OPT_n
  index.use_normal = true
else
  index.use_table = true
end

if $OPT_d and FileTest.directory?($OPT_d)
  message = index.check_essential_files($OPT_d)
  if message.size > 0
    message.each{|e|
      printf "  %s does not exist!\n", e
    }
    exit
  end
  index.run($OPT_d)
end

